from data import *
from core.three_d.Svg3DPanel import *
from core.styles import atom_colors

class MoleculePanel(Svg3DPanel):
    """Represenst a panel to display chemical molecules
    """

    def __init__(self, svg_canvas, width=500, height=500):
        super().__init__(svg_canvas, width, height)

    def set_molecule(self, atms, bonds, flag="H", s=40):

        self.scale = s
        """center of mass"""

        suma_x = 0
        suma_y = 0
        suma_z = 0

        for ai in atms:
            suma_x += ai.x
            suma_y += ai.y
            suma_z += ai.z

        self.center_x = suma_x / float(len(atms))
        self.center_y = suma_y / float(len(atms))
        self.center_z = suma_z / float(len(atms))

        for atom in atom_colors.keys():
          self.define_gradient("grad%s" % atom, ["0%", "white", "0.9"], ["100%", "%s" % atom_colors[atom], "1"])

        i = 0
        for ai in atms:
            
            c_x, c_y = ai[0] *self.scale-self.center_x * self.scale, ai[1] *self.scale-self.center_y * self.scale
            c_gradient = "grad%s" % ai.element
            c_r = vdw_atomic_radii[ai.element] * 5
            if flag=="H":
              self.add_vertex(c_x,c_y,ai[2] * self.scale-self.center_z * self.scale)
              
              self.sphere_at("circle"+str(i),i,c_r,fill=atom_colors[ai.element], stroke_width=0, gradient=c_gradient)
              i += 1
            else:
              if ai.element != "H":
                self.add_vertex(c_x,c_y,ai[2] * self.scale-self.center_z * self.scale)
                self.sphere_at("circle"+str(i),i,c_r,fill=atom_colors[ai.element], stroke_width=0, gradient=c_gradient)
                i += 1

        i = 0
        for li in bonds:
            ind1 = li.atom1.id - 1
            ind2 = li.atom2.id - 1

            if int(li.type) >= 2:
                self.line_at("line" + str(i), ind1, ind2, stroke='black', stroke_width='5')
                self.line_at("doubleline" + str(i), ind1, ind2,  stroke='white', stroke_width='2')
            else:
                self.line_at("line" + str(i), ind1, ind2, stroke='black', stroke_width='2')
            i += 1
    

C2H5OH = """702
  -OEChem-02271511112D

  9  8  0     0  0  0  0  0  0999 V2000
    0.5369    0.9749    0.0000 O   0  0  0  0  0  0  0  0  0  0  0  0
    1.4030    0.4749    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
    2.2690    0.9749    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
    1.8015    0.0000    0.0000 H   0  0  0  0  0  0  0  0  0  0  0  0
    1.0044    0.0000    0.0000 H   0  0  0  0  0  0  0  0  0  0  0  0
    1.9590    1.5118    0.0000 H   0  0  0  0  0  0  0  0  0  0  0  0
    2.8059    1.2849    0.0000 H   0  0  0  0  0  0  0  0  0  0  0  0
    2.5790    0.4380    0.0000 H   0  0  0  0  0  0  0  0  0  0  0  0
    0.0000    0.6649    0.0000 H   0  0  0  0  0  0  0  0  0  0  0  0
  1  2  1  0  0  0  0
  1  9  1  0  0  0  0
  2  3  1  0  0  0  0
  2  4  1  0  0  0  0
  2  5  1  0  0  0  0
  3  6  1  0  0  0  0
  3  7  1  0  0  0  0
  3  8  1  0  0  0  0
M  END"""

CFF="""CFF
  CCTOOLS-0615191158

 24 25  0  0  0  0  0  0  0  0999 V2000
    1.0470   -0.0000   -1.3120 N   0  0  0  0  0
   -0.2080   -0.0000   -1.7900 C   0  0  0  0  0
    2.1760    0.0000   -2.2460 C   0  0  0  0  0
    1.2850   -0.0010    0.0160 C   0  0  0  0  0
   -1.2760   -0.0000   -0.9710 N   0  0  0  0  0
   -0.3840    0.0000   -2.9930 O   0  0  0  0  0
   -2.6290   -0.0000   -1.5330 C   0  0  0  0  0
   -1.0980   -0.0000    0.4020 C   0  0  0  0  0
    0.1930    0.0050    0.9110 C   0  0  0  0  0
   -1.9340   -0.0000    1.4440 N   0  0  0  0  0
    2.4280   -0.0000    0.4370 O   0  0  0  0  0
    0.0680   -0.0000    2.2860 N   0  0  0  0  0
   -1.2510   -0.0000    2.5600 C   0  0  0  0  0
    1.1610   -0.0000    3.2610 C   0  0  0  0  0
    1.8000    0.0010   -3.2690 H   0  0  0  0  0
    2.7830    0.8900   -2.0820 H   0  0  0  0  0
    2.7830   -0.8890   -2.0830 H   0  0  0  0  0
   -2.5700   -0.0000   -2.6220 H   0  0  0  0  0
   -3.1620   -0.8900   -1.1980 H   0  0  0  0  0
   -3.1620    0.8890   -1.1980 H   0  0  0  0  0
   -1.6790    0.0000    3.5520 H   0  0  0  0  0
    1.4320   -1.0280    3.5030 H   0  0  0  0  0
    2.0240    0.5130    2.8390 H   0  0  0  0  0
    0.8390    0.5130    4.1670 H   0  0  0  0  0
  1  2  1  0  0  0
  1  3  1  0  0  0
  1  4  1  0  0  0
  2  5  1  0  0  0
  2  6  2  0  0  0
  3 15  1  0  0  0
  3 16  1  0  0  0
  3 17  1  0  0  0
  4  9  1  0  0  0
  4 11  2  0  0  0
  5  7  1  0  0  0
  5  8  1  0  0  0
  7 18  1  0  0  0
  7 19  1  0  0  0
  7 20  1  0  0  0
  8  9  2  0  0  0
  8 10  1  0  0  0
  9 12  1  0  0  0
 10 13  2  0  0  0
 12 13  1  0  0  0
 12 14  1  0  0  0
 13 21  1  0  0  0
 14 22  1  0  0  0
 14 23  1  0  0  0
 14 24  1  0  0  0
M  END
"""

NIK="""ZINC00391812
  -OEChem-06231903453D

 27 28  0     1  0  0  0  0  0999 V2000
    0.3944    1.5372   -0.0882 C   0  0  0  0  0  0  0  0  0  0  0  0
    0.2218    0.0783   -0.0924 N   0  3  1  0  0  0  0  0  0  0  0  0
    0.9063   -0.4989   -1.2638 C   0  0  0  0  0  0  0  0  0  0  0  0
    1.9925   -1.4575   -0.7321 C   0  0  0  0  0  0  0  0  0  0  0  0
    2.2411   -0.9393    0.7089 C   0  0  0  0  0  0  0  0  0  0  0  0
    0.8181   -0.4968    1.1321 C   0  0  1  0  0  0  0  0  0  0  0  0
    0.2382   -1.3559    1.4692 H   0  0  0  0  0  0  0  0  0  0  0  0
    0.8940    0.5453    2.2181 C   0  0  0  0  0  0  0  0  0  0  0  0
    1.8431    1.5545    2.1577 C   0  0  0  0  0  0  0  0  0  0  0  0
    1.8760    2.4913    3.1790 C   0  0  0  0  0  0  0  0  0  0  0  0
    0.9680    2.3891    4.2162 C   0  0  0  0  0  0  0  0  0  0  0  0
    0.0793    1.4154    4.2410 N   0  0  0  0  0  0  0  0  0  0  0  0
    0.0150    0.5105    3.2837 C   0  0  0  0  0  0  0  0  0  0  0  0
    1.4576    1.7772   -0.0812 H   0  0  0  0  0  0  0  0  0  0  0  0
   -0.0678    1.9603   -0.9800 H   0  0  0  0  0  0  0  0  0  0  0  0
   -0.0786    1.9564    0.7999 H   0  0  0  0  0  0  0  0  0  0  0  0
    0.1926   -1.0502   -1.8760 H   0  0  0  0  0  0  0  0  0  0  0  0
    1.3676    0.2945   -1.8519 H   0  0  0  0  0  0  0  0  0  0  0  0
    1.6257   -2.4838   -0.7146 H   0  0  0  0  0  0  0  0  0  0  0  0
    2.9001   -1.3816   -1.3309 H   0  0  0  0  0  0  0  0  0  0  0  0
    2.6094   -1.7378    1.3529 H   0  0  0  0  0  0  0  0  0  0  0  0
    2.9285   -0.0933    0.7065 H   0  0  0  0  0  0  0  0  0  0  0  0
    2.5404    1.6091    1.3347 H   0  0  0  0  0  0  0  0  0  0  0  0
    2.6020    3.2907    3.1646 H   0  0  0  0  0  0  0  0  0  0  0  0
    0.9870    3.1149    5.0157 H   0  0  0  0  0  0  0  0  0  0  0  0
   -0.7302   -0.2695    3.3356 H   0  0  0  0  0  0  0  0  0  0  0  0
   -0.7608   -0.1480   -0.1301 H   0  0  0  0  0  0  0  0  0  0  0  0
  1  2  1  0  0  0  0
  1 14  1  0  0  0  0
  1 15  1  0  0  0  0
  1 16  1  0  0  0  0
  2  6  1  0  0  0  0
  2  3  1  0  0  0  0
  2 27  1  0  0  0  0
  3  4  1  0  0  0  0
  3 17  1  0  0  0  0
  3 18  1  0  0  0  0
  4  5  1  0  0  0  0
  4 19  1  0  0  0  0
  4 20  1  0  0  0  0
  5  6  1  0  0  0  0
  5 21  1  0  0  0  0
  5 22  1  0  0  0  0
  6  7  1  0  0  0  0
  6  8  1  0  0  0  0
  8 13  2  0  0  0  0
  8  9  1  0  0  0  0
  9 10  2  0  0  0  0
  9 23  1  0  0  0  0
 10 11  1  0  0  0  0
 10 24  1  0  0  0  0
 11 12  2  0  0  0  0
 11 25  1  0  0  0  0
 12 13  1  0  0  0  0
 13 26  1  0  0  0  0
M  CHG  1   2   1
M  END
"""

LDP = """LDP
  CCTOOLS-0615191208

 22 22  0  0  0  0  0  0  0  0999 V2000
    0.0380   -0.7070    2.1700 C   0  0  0  0  0
    0.1890   -0.4110    0.7010 C   0  0  0  0  0
    0.4720    0.1330   -1.9990 C   0  0  0  0  0
   -0.9320   -0.2990   -0.0980 C   0  0  0  0  0
    1.4490   -0.2560    0.1540 C   0  0  0  0  0
    1.5930    0.0150   -1.1930 C   0  0  0  0  0
   -0.7960   -0.0220   -1.4490 C   0  0  0  0  0
   -1.8990    0.0920   -2.2360 O   0  0  0  0  0
    0.6110    0.3990   -3.3260 O   0  0  0  0  0
   -0.0630    0.6060    2.9480 C   0  0  0  0  0
   -0.2100    0.3180    4.3800 N   0  0  0  0  0
    0.9050   -1.2680    2.5190 H   0  0  0  0  0
   -0.8640   -1.2960    2.3320 H   0  0  0  0  0
   -1.9160   -0.4210    0.3310 H   0  0  0  0  0
    2.3240   -0.3480    0.7810 H   0  0  0  0  0
    2.5780    0.1350   -1.6180 H   0  0  0  0  0
   -2.0920   -0.7910   -2.5770 H   0  0  0  0  0
    0.6730   -0.4520   -3.7770 H   0  0  0  0  0
   -0.9300    1.1680    2.5990 H   0  0  0  0  0
    0.8390    1.1950    2.7860 H   0  0  0  0  0
   -0.2740    1.2080    4.8510 H   0  0  0  0  0
   -1.1050   -0.1340    4.4950 H   0  0  0  0  0
  1  2  1  0  0  0
  1 10  1  0  0  0
  1 12  1  0  0  0
  1 13  1  0  0  0
  2  4  2  0  0  0
  2  5  1  0  0  0
  3  6  1  0  0  0
  3  7  2  0  0  0
  3  9  1  0  0  0
  4  7  1  0  0  0
  4 14  1  0  0  0
  5  6  2  0  0  0
  5 15  1  0  0  0
  6 16  1  0  0  0
  7  8  1  0  0  0
  8 17  1  0  0  0
  9 18  1  0  0  0
 10 11  1  0  0  0
 10 19  1  0  0  0
 10 20  1  0  0  0
 11 21  1  0  0  0
 11 22  1  0  0  0
M  END
"""

TES="""TES
  CCTOOLS-0615191210

 49 52  0  0  1  0  0  0  0  0999 V2000
    1.5950    0.1910   -2.4880 C   0  0  0  0  0
    1.3460   -0.9130   -3.5130 C   0  0  0  0  0
    0.1350   -0.5130   -4.3340 C   0  0  0  0  0
    0.1610   -0.4850   -5.5470 O   0  0  0  0  0
   -1.0710   -0.1620   -3.5730 C   0  0  0  0  0
   -0.9410    0.2800   -2.3260 C   0  0  0  0  0
   -2.1930    0.6010   -1.5270 C   0  0  0  0  0
   -2.1140   -0.1740   -0.2070 C   0  0  0  0  0
   -0.8100    0.1600    0.5170 C   0  0  0  0  0
    0.3910   -0.2930   -0.3280 C   0  0  0  0  0
    0.3600    0.4850   -1.6430 C   0  0  0  0  0
    1.7120   -0.0340    0.3880 C   0  0  0  0  0
    1.7400   -0.6840    1.7810 C   0  0  0  0  0
    0.5630   -0.1270    2.5590 C   0  0  0  0  0
   -0.7280   -0.6060    1.8430 C   0  0  0  0  0
   -1.8080   -0.2680    2.8760 C   0  0  0  0  0
   -1.1520   -0.6930    4.2180 C   0  0  0  0  0
    0.3880   -0.6370    3.9850 C   0  0  0  0  0
    0.9990    0.2470    4.9270 O   0  0  0  0  0
    0.6190    1.4020    2.5590 C   0  0  0  0  0
    0.4240    1.9680   -1.2740 C   0  0  0  0  0
    2.4120   -0.1110   -1.8330 H   0  0  0  0  0
    1.8840    1.1010   -3.0130 H   0  0  0  0  0
    1.1520   -1.8560   -3.0020 H   0  0  0  0  0
    2.2160   -1.0170   -4.1630 H   0  0  0  0  0
   -2.0500   -0.2630   -4.0200 H   0  0  0  0  0
   -3.0770    0.2920   -2.0850 H   0  0  0  0  0
   -2.2380    1.6710   -1.3240 H   0  0  0  0  0
   -2.1480   -1.2440   -0.4140 H   0  0  0  0  0
   -2.9590    0.0970    0.4230 H   0  0  0  0  0
   -0.7520    1.2350    0.6880 H   0  0  0  0  0
    0.3000   -1.3590   -0.5390 H   0  0  0  0  0
    1.8540    1.0410    0.4940 H   0  0  0  0  0
    2.5270   -0.4410   -0.2100 H   0  0  0  0  0
    2.6720   -0.4350    2.2880 H   0  0  0  0  0
    1.6440   -1.7650    1.6870 H   0  0  0  0  0
   -0.7000   -1.6800    1.6590 H   0  0  0  0  0
   -2.0240    0.7990    2.8740 H   0  0  0  0  0
   -2.7140   -0.8460    2.6890 H   0  0  0  0  0
   -1.4310    0.0030    5.0080 H   0  0  0  0  0
   -1.4640   -1.7030    4.4830 H   0  0  0  0  0
    0.8190   -1.6340    4.0700 H   0  0  0  0  0
    0.8230   -0.1150    5.8060 H   0  0  0  0  0
    0.6600    1.7640    1.5310 H   0  0  0  0  0
   -0.2690    1.7980    3.0490 H   0  0  0  0  0
    1.5080    1.7330    3.0960 H   0  0  0  0  0
    0.4250    2.5690   -2.1840 H   0  0  0  0  0
   -0.4410    2.2290   -0.6670 H   0  0  0  0  0
    1.3360    2.1630   -0.7100 H   0  0  0  0  0
  1  2  1  0  0  0
  1 11  1  0  0  0
  1 22  1  0  0  0
  1 23  1  0  0  0
  2  3  1  0  0  0
  2 24  1  0  0  0
  2 25  1  0  0  0
  3  4  2  0  0  0
  3  5  1  0  0  0
  5  6  2  0  0  0
  5 26  1  0  0  0
  6  7  1  0  0  0
  6 11  1  0  0  0
  7  8  1  0  0  0
  7 27  1  0  0  0
  7 28  1  0  0  0
  8  9  1  0  0  0
  8 29  1  0  0  0
  8 30  1  0  0  0
  9 10  1  0  0  0
  9 15  1  0  0  0
  9 31  1  0  0  0
 10 11  1  0  0  0
 10 12  1  0  0  0
 10 32  1  0  0  0
 11 21  1  0  0  0
 12 13  1  0  0  0
 12 33  1  0  0  0
 12 34  1  0  0  0
 13 14  1  0  0  0
 13 35  1  0  0  0
 13 36  1  0  0  0
 14 15  1  0  0  0
 14 18  1  0  0  0
 14 20  1  0  0  0
 15 16  1  0  0  0
 15 37  1  0  0  0
 16 17  1  0  0  0
 16 38  1  0  0  0
 16 39  1  0  0  0
 17 18  1  0  0  0
 17 40  1  0  0  0
 17 41  1  0  0  0
 18 19  1  0  0  0
 18 42  1  0  0  0
 19 43  1  0  0  0
 20 44  1  0  0  0
 20 45  1  0  0  0
 20 46  1  0  0  0
 21 47  1  0  0  0
 21 48  1  0  0  0
 21 49  1  0  0  0
M  END
"""
